package com.ippon.error.domain;

public final class Assert {

  private Assert() {}

  public static void notNull(String field, Object input) {
    if (input == null) {
      throw MissingMandatoryValueException.forNullValue(field);
    }
  }

  public static void notBlank(String field, String input) {
    notNull(field, input);

    if (input.isBlank()) {
      throw MissingMandatoryValueException.forBlankValue(field);
    }
  }

  public static LongAsserter field(String fieldName, long value) {
    return new LongAsserter(fieldName, value);
  }

  public static class LongAsserter {
    private final String fieldName;
    private final long value;

    private LongAsserter(String fieldName, long value) {
      this.fieldName = fieldName;
      this.value = value;
    }

    public LongAsserter min(long minValue) {
      if (value < minValue) {
        throw ValueUnderMinException.builder().field(fieldName).value(value).minValue(minValue).build();
      }

      return this;
    }

    public LongAsserter max(long maxValue) {
      if (value > maxValue) {
        throw ValueOverMaxException.builder().field(fieldName).value(value).maxValue(maxValue).build();
      }

      return this;
    }
  }
}
