package com.ippon.pagination.domain;

import static org.assertj.core.api.Assertions.*;

import com.ippon.error.domain.ValueOverMaxException;
import com.ippon.error.domain.ValueUnderMinException;
import org.junit.jupiter.api.Test;

class MyAppPageableUnitTest {

  @Test
  void shouldNotBuildWithNegativePage() {
    assertThatThrownBy(() -> new MyAppPageable(-1, 10)).isExactlyInstanceOf(ValueUnderMinException.class).hasMessageContaining("page");
  }

  @Test
  void shouldNotBuildWithPageSizeAtZero() {
    assertThatThrownBy(() -> new MyAppPageable(0, 0)).isExactlyInstanceOf(ValueUnderMinException.class).hasMessageContaining("pageSize");
  }

  @Test
  void shouldNotBuildWithPageSizeOverHundred() {
    assertThatThrownBy(() -> new MyAppPageable(0, 101)).isExactlyInstanceOf(ValueOverMaxException.class).hasMessageContaining("pageSize");
  }

  @Test
  void shouldNotPaginateOverTenThousand() {
    assertThatThrownBy(() -> new MyAppPageable(101, 100))
      .isExactlyInstanceOf(PaginationException.class)
      .hasMessageContaining("over 10 000");
  }

  @Test
  void shouldGetFirstPageInformation() {
    MyAppPageable pageable = new MyAppPageable(0, 15);

    assertThat(pageable.getPage()).isEqualTo(0);
    assertThat(pageable.getPageSize()).isEqualTo(15);
    assertThat(pageable.getOffset()).isEqualTo(0);
  }

  @Test
  void shouldGetPageableInformation() {
    MyAppPageable pageable = new MyAppPageable(2, 15);

    assertThat(pageable.getPage()).isEqualTo(2);
    assertThat(pageable.getPageSize()).isEqualTo(15);
    assertThat(pageable.getOffset()).isEqualTo(30);
  }
}
