package com.ippon.pagination.infrastructure.primary;

import static com.ippon.BeanValidationAssertions.*;
import static org.assertj.core.api.Assertions.*;

import com.ippon.error.domain.ValidationMessage;
import com.ippon.pagination.domain.MyAppPageable;
import org.junit.jupiter.api.Test;

class RestMyAppPageableUnitTest {

  @Test
  void shouldConvertToDomain() {
    MyAppPageable pageable = pageable().toPageable();

    assertThat(pageable.getPage()).isEqualTo(1);
    assertThat(pageable.getPageSize()).isEqualTo(15);
  }

  @Test
  void shouldNotValidateWithPageUnderZero() {
    RestMyAppPageable pageable = pageable();
    pageable.setPage(-1);

    assertThatBean(pageable).hasInvalidProperty("page").withMessage(ValidationMessage.VALUE_TOO_LOW);
  }

  @Test
  void shouldNotValidateWithSizeAtZero() {
    RestMyAppPageable pageable = pageable();
    pageable.setPageSize(0);

    assertThatBean(pageable).hasInvalidProperty("pageSize").withMessage(ValidationMessage.VALUE_TOO_LOW).withArgument("value", 1L);
  }

  @Test
  void shouldNotValidateWithPageSizeOverHundred() {
    RestMyAppPageable pageable = pageable();
    pageable.setPageSize(101);

    assertThatBean(pageable).hasInvalidProperty("pageSize").withMessage(ValidationMessage.VALUE_TOO_HIGH);
  }

  private RestMyAppPageable pageable() {
    RestMyAppPageable pageable = new RestMyAppPageable();

    pageable.setPage(1);
    pageable.setPageSize(15);

    return pageable;
  }
}
